//************************************************************************
//This module contains fitness case and functions to compute raw fitness *
//as well as standardized fitness for Symbolic Regression Problem        *
//Programmed by NXH.                                                     *
//************************************************************************
#ifndef __SYMBOLICREG__
#define __SYMBOLICREG__

#include<iostream.h>
#include<string.h>
#include<stdio.h>
#include<assert.h>
#include<math.h>
#include<stdlib.h>
#include "gconst.h"
#include "random.h"
#include "term.h"
#include "stree.h"
#include "genetic.h"

#define NUMFITCASE 50
#define INFPLUS  exp(700)
#define INFMINUS -exp(700)
#define PVAL(y) ((y==HUGE_VAL)?(INFPLUS):((y==-HUGE_VAL)?(-INFMINUS):(y)))
//****************************************
//Fitness cases for symbolic regression  *
//They are sampled between 0  and 1      *
//Seed for sampling 12345                *
//****************************************
struct _sample{
		 double x;
		 double y;
		};
typedef struct _sample sample;
sample fitcase[NUMFITCASE];
BOOL SuccPredicate=FALSE;
void SetFitCase()
{
 //FILE *fp;
  double x;
  int i;
  for(i=0;i<NUMFITCASE;i++)
   {
     fitcase[i].x=-1+i*(1/(double)NUMFITCASE);
     x=fitcase[i].x;
     fitcase[i].y=(x+1)*(x+1)*(x+1);
   }
 // fclose(fp);
}
//********************************************************************
//This function compute the function (in tree form) with given input *
//********************************************************************
double Compute(node *st,double x)
{
 node *p;
 double l,r;
 if (!strcmp(st->name,"X"))
     return x;
 else 
  {
    p=st->children;
    if (!strcmp(st->name,"add"))
    {
     l=Compute(p,x);
     r=Compute(p->sibling,x);
     return PVAL(l+r);
    }
   else
   if (!strcmp(st->name,"sub"))
    {
     l=Compute(p,x);
     r=Compute(p->sibling,x);
     return PVAL(l-r);
    }
   else
   if (!strcmp(st->name,"mul"))
    {
     l=Compute(p,x);
     r=Compute(p->sibling,x);
     return PVAL(l*r);
    }
    else
    {
     l=Compute(p,x);
     r=Compute(p->sibling,x);
    if(r==0)
     return 1;
    else
     return PVAL(l/r);
    }
  }
}
//*********************************************
//Compute the raw fitness for an invidual     *
//*********************************************
double ComputeRF(individual st)
{
  double sum=0,t;
  int i;
  int hit;
  hit=0;
  for(i=0;i<NUMFITCASE;i++)
   {
     t=PVAL(fabs(Compute(st.chrom,fitcase[i].x)-fitcase[i].y));
     if(t<=0.01)
       hit++;
     sum+=t;
   }
   if(hit==NUMFITCASE)
     SuccPredicate=TRUE;
   return sum;
}
//****************************************************************
//This function initializes elementary trees and adjoining table *
//****************************************************************
void LoadTerm()
{
 cout<<endl<<"Loading terms...";
 InitTerm();
 AddTerm("add",2);
 AddTerm("sub",2);
 AddTerm("div",2);
 AddTerm("mul",2);
 AddTerm("X",0);
// SwapTerm(100);
 cout<<endl<<"Done!";
}
#endif
